'''
purpose: take images of sea turtle plastrons and quantify (in number of pixels) the amount of
         deketatinization.
         
design:  We use a model-view-controller triad of classes to implement the plastron analysis application.
         The model is the application object.  The view is the screen presentation and the controller defines 
         the way the GUI reacts to user input.

method:  (1) calculate the number of pixes that make up the entire plastron
         (2) calculate the number of pixes that make up the dekeratinized area
         (3) take the ratio of the two to come up with a relative estimate of dekeratinization

author:  A. Richards 2/08 richa@musc.edu

'''

#from numpy import array, where
import sys


class Model:
    def __init__(self, master,controller):
        self.restOfPoints = []
        self.points1 = []
        self.points2 = []
        self.newLine = True
        self.controller = controller
        self.lines = {}
        self.eraser = False
        self.lineType = None

    def get_new_ycoords(self):

        self.ycoords = {}
        for i in range(800 + 1):
            self.ycoords[i] = []

    def press(self,event):
        self.lineStart = event.x, event.y
        self.newLine = True
        
    def get_rest_of_points(self, point1, point2):
        restOfPoints = [] 
        estimateX = point1[0]
        estimateY = point1[1]
        
        stepY = 1
        stepX = 1

        ## iterate to get points
        while (estimateX, estimateY) != point2:
            if estimateX == point2[0]:
                pass
            elif estimateX < point2[0]:
                estimateX += stepX
            elif estimateX > point2[0]:
                estimateX -= stepX
                estimateX = round(estimateX)

            if estimateY == point2[1]:
                pass
            elif estimateY < point2[1]:
                estimateY += stepY
            elif estimateY > point2[1]:
                estimateY -= stepY
                estimateY = round(estimateY)

            restOfPoints.append((estimateX, estimateY))

        return restOfPoints

    def motion(self,event):

        if self.newLine is True:
            self.points2 = self.controller.view.canvas.canvasx(event.x),self.controller.view.canvas.canvasy(event.y)     
            self.newLine = False

        else: 
            self.points1 = self.points2
            self.points2 = self.controller.view.canvas.canvasx(event.x),self.controller.view.canvas.canvasy(event.y)

            if self.eraser == False:
              
                if self.lineType is 'plastron':
                    self.controller.pb_tag += 1
                    item = self.controller.view.canvas.create_line(self.points1,self.points2,fill=self.color,width=2)  
                    self.lines[item] = "pb" + str(self.controller.pb_tag) 

                elif self.lineType is 'softness':
                    self.controller.sb_tag += 1
                    item = self.controller.view.canvas.create_line(self.points1,self.points2,fill=self.color,width=2)
                    self.lines[item] = "sb" + str(self.controller.sb_tag)
                else:
                    print "ERROR: lineType not specified"
                    sys.exit()
            else:
                
                items = self.controller.view.canvas.find_overlapping(self.points1[0],self.points1[1],                                                                                           self.points2[0],self.points2[1])
               
                for item in items:
                    if self.lines.has_key(item):
                        key = self.lines[item]

                        if self.lineType == 'plastron':
                            if key[:2] == 'pb' or key[:2] == 'pf': 
                                self.controller.view.canvas.delete(item)

                        elif self.lineType == 'softness':
                            if key[:2] == 'sb' or key[:2] == 'sf': 
                                self.controller.view.canvas.delete(item)
                        else:
                            print "ERROR: lineType not specified"
                            sys.exit()
                        

    def release(self, event):
        self.points2 = event.x, event.y
        self.points1 = self.points2
