'''
purpose: take images of sea turtle plastrons and quantify (in number of pixels) the amount of
         deketatinization.
         
design:  We use a model-view-controller triad of classes to implement the plastron analysis application.
         The model is the application object.  The view is the screen presentation and the controller defines 
         the way the GUI reacts to user input.



method:  (1) calculate the number of pixes that make up the entire plastron
         (2) calculate the number of pixes that make up the dekeratinized area
         (3) take the ratio of the two to come up with a relative estimate of dekeratinization

author:  A. Richards 2/08 richa@musc.edu

'''

from PIL import Image,ImageTk
from Tkinter import *

class View:

    def __init__(self,master,controller):

        import Image
        self.master = master
        self.cWidth = 200    # canvas width
        self.cHeight = 650   # canvas height

        ### create the top area
        self.topArea = Frame(master)
        self.topArea.pack()

        self.b0 = IntVar()
        self.b0.set(1)
        self.button0=Checkbutton(self.topArea,fg="black",command=controller.return_pointer,variable = self.b0)
        self.button0["text"] = "Pointer"
        self.button0["background"] = "tan"
        self.button0.pack(side=LEFT)

        self.b1 = IntVar()
        self.b1.set(0)
        self.button1=Checkbutton(self.topArea,fg="red",command=controller.outline_plastron,variable = self.b1)
        self.button1["text"] = "Outline Plastron"      
        self.button1["background"] = "black"
        self.button1.pack(side=LEFT)
        
        self.button2 = Button(self.topArea, text="Set Area", fg="red", command = controller.set_plastron_area)
        self.button2.pack(side=LEFT)
        self.button2["background"] = "black"

        self.b3 = IntVar()
        self.b3.set(0)
        self.button3=Checkbutton(self.topArea,fg="green",command=controller.outline_softness,variable = self.b3)
        self.button3["text"] = "Outline Softness"
        self.button3["background"] = "black"
        self.button3.pack(side=LEFT)
        
        self.button4 = Button(self.topArea, text="Set Area", fg="green", command = controller.set_softness_area)
        self.button4.pack(side=LEFT)
        self.button4["background"] = "black"

        self.b5 = IntVar()
        self.b5.set(0)
        self.button5 = Checkbutton(self.topArea,text="Eraser",fg="black",command=controller.eraser,variable = self.b5)
        self.button5.pack(side=LEFT)
        self.button5["background"] = "tan"
        
        ### create menu
        self.menu = Menu(master)
        self.master = master
        self.master.config(menu=self.menu)
        self.filemenu = Menu(self.menu)
        self.menu.add_cascade(label="File", menu = self.filemenu)

        #self.filemenu.add_command(label="Create new project", command = self.callback)
        self.filemenu.add_command(label="Save project as", command = controller.saveas_data)
        self.filemenu.add_command(label="Save project", command = controller.save_data)
        self.filemenu.add_command(label="Load image", command = controller.load_image)
        self.filemenu.add_command(label="Save image as", command = controller.saveas_image)
        self.filemenu.add_separator()
        self.filemenu.add_command(label="Exit", command = self.topArea.quit)

        self.helpmenu = Menu(self.menu)
        self.menu.add_cascade(label="Help", menu = self.helpmenu)
        self.helpmenu.add_command(label="User Manual", command = controller.show_documentation)
        self.helpmenu.add_command(label="About...", command = self.callback)

        ### create the right side
        self.rightAreaFrame = Frame(master)
        self.rightAreaFrame["bg"] = "black"
        self.rightArea = Canvas(self.rightAreaFrame,width=200,height=self.cHeight)#800
        self.rightArea.configure(background = "black")

        self.rightArea.button_date = Button(self.rightAreaFrame, fg="black", command = controller.enter_date)
        self.rightArea.button_date["background"] = "tan"
        self.rightArea.button_date["text"] = "Enter Date"
        
        self.rightArea.button_id = Button(self.rightAreaFrame,fg="black",command=controller.enter_id)
        self.rightArea.button_id["text"] = "Enter Turtle ID"
        self.rightArea.button_id["background"] = "tan"
       
        self.rightArea.button_pid = Button(self.rightAreaFrame,fg="black",command=controller.enter_pid)
        self.rightArea.button_pid["text"] = "Enter Project Name"
        self.rightArea.button_pid["background"] = "tan"

        self.rightAreaFrame.pack(side = RIGHT)
        self.rightArea.button_id.pack()
        self.rightArea.button_pid.pack()
        self.rightArea.button_date.pack()
        self.rightArea.pack()
        self.rightArea.create_text(100, 640, text="PLASTron (Version: 1.0)", fill = "white")

        ### create the center
        self.centerAreaFrame = Frame(master)
        self.centerAreaFrame.pack(side = RIGHT)
        self.centerAreaFrame["bg"] = "black"
        self.canvas = Canvas(self.centerAreaFrame,width = 820, height = self.cHeight)#800
        self.canvas.configure(background = "tan")
        from PIL import Image,ImageTk
        im = Image.open("green_sea_turtle.png")
        im.thumbnail((800,600),Image.ANTIALIAS)
        self.i = ImageTk.PhotoImage(im)
        self.canvas.create_image(410,400,image = self.i, anchor = CENTER, tags = "photo")
        self.canvas.create_text(410, 20, text="PLASTron (Version: 1.0)", tags = "title1")
        self.canvas.create_text(410, 35, text="PLastron Analysis Software Tool", tags = "title2")
        self.canvas.pack()

        ### create the right side 
        self.leftAreaFrame = Frame(master)
        self.leftAreaFrame["bg"] = "black"
        self.messageBoard = Canvas(self.leftAreaFrame,width=200,height = self.cHeight)#800
        self.messageBoard.configure(background = "black")
        
        self.messageBoard.button_cp = Button(self.leftAreaFrame, fg="black", command = controller.clear_plastron)
        self.messageBoard.button_cp["background"] = "tan"
        self.messageBoard.button_cp["text"] = "Clear plastron"
      
        self.messageBoard.button_cs = Button(self.leftAreaFrame, fg="black", command = controller.clear_softness)
        self.messageBoard.button_cs["background"] = "tan"
        self.messageBoard.button_cs["text"] = "Clear softness"
      
        self.messageBoard.button_ca = Button(self.leftAreaFrame, fg="black", command = controller.clear_all)
        self.messageBoard.button_ca["background"] = "tan"
        self.messageBoard.button_ca["text"] = "Clear all"
      
        self.messageBoard.create_line((0,15),(200,15), fill = "white")
        self.messageBoard.create_text(100, 10, text="Results", fill = "white")
       
        self.leftAreaFrame.pack(side = RIGHT)
        self.messageBoard.button_cp.pack()
        self.messageBoard.button_cs.pack()
        self.messageBoard.button_ca.pack()
        self.messageBoard.pack()
        
    def mount_image(self,fileName):
        #self.i = PhotoImage(file=fileName)
        self.i = ImageTk.PhotoImage(fileName)
        self.canvas.delete("photo")
        self.canvas.create_image(410,347,image=self.i, anchor = CENTER, tags = "photo")

    def callback(self):
        print "Need to make this function"


class StatusBar(Frame):

    def __init__(self, master, controller):
        self.frame = Frame.__init__(self, master)
        self.label = Label(self, bd=1, relief=SUNKEN, anchor=W)
        self.label.config(text = "load an image to get started")
        self.label.pack(fill=X)
        self.pack(side = BOTTOM)
        self.button9 = Button(self.frame, text="Calculate", fg="white", command = controller.calculate_relative_area)
        self.button9["padx"] = 10.0
        self.button9["pady"] = 5.0
        self.button9.pack(side = BOTTOM)
        self.button9["background"] = "black"

    def set(self, msg):
        self.label.config(text = msg)
        self.label.update_idletasks()

    def clear(self):
        self.label.config(text="")
        self.label.update_idletasks()
