'''
purpose: take images of sea turtle plastrons and quantify (in number of pixels) the amount of
         deketatinization.
         
design:  We use a model-view-controller triad of classes to implement the plastron analysis application.
         The model is the application object.  The view is the screen presentation and the controller defines 
         the way the GUI reacts to user input.

method:  (1) calculate the number of pixes that make up the entire plastron
         (2) calculate the number of pixes that make up the dekeratinized area
         (3) take the ratio of the two to come up with a relative estimate of dekeratinization

author:  A. Richards 2/08 richa@musc.edu

'''

from PIL import Image
import re
from numpy import array,where
from Tkinter import *
import tkMessageBox,tkSimpleDialog
from tkFileDialog import askopenfilename, asksaveasfilename, SaveAs
import os,sys
from Model import Model
from View import View, StatusBar

class Controller:
    def __init__(self):
        self.root = Tk()
        self.currentFileName = None
        self.set_variables()
        self.status = StatusBar(self.root, self)
        self.view = View(self.root, self)
        self.model = Model(self.root,self)
        self.openScreen = True
        self.root.mainloop()

    def set_variables(self):
        self.date = None
        self.project_id = None
        self.turtle_id = None

        self.plastronCoords = {}
        self.softnessCoords = {}
        self.pf_tag = 0           # plastron fill tag
        self.pb_tag = 0           # plastron border tag
        self.sf_tag = 0           # softness fill tag
        self.sb_tag = 0           # softness border tag

        self.plastronArea = 0
        self.softnessArea = 0
        self.relativeArea = 0

    def outline_plastron(self):
        if self.openScreen is False:
            if self.view.b1.get() == 1:
                self.view.canvas.configure(cursor = "arrow")
                self.model.eraser = False
                self.model.lineType = 'plastron'
                self.view.b0.set(0)
                self.view.b3.set(0)
                self.view.b5.set(0)
                self.view.canvas.unbind("<ButtonPress-1>")
                self.view.canvas.unbind("<B1-Motion>")
                self.view.canvas.unbind("<ButtonRelease-1>")
                self.model.color = 'red'
                self.view.canvas.bind("<ButtonPress-1>", self.model.press)
                self.view.canvas.bind("<B1-Motion>", self.model.motion)
                self.view.canvas.bind("<ButtonRelease-1>", self.model.release)
                self.status.set("Next: 'Set area'")
            elif self.view.b1.get() == 0:
                self.view.b0.set(1)
                self.return_pointer()
        else:
            self.status.set("Please load an image first")
            tkMessageBox.showinfo("PLASTron    ","Please load an image first")
            self.view.b1.set(0)

    def outline_softness(self):
        if self.openScreen is False:      
            if self.view.b3.get() == 1:
                self.view.canvas.configure(cursor = "arrow")
                self.model.eraser = False
                self.model.lineType = 'softness'
                self.view.b0.set(0)
                self.view.b1.set(0)
                self.view.b5.set(0)
                self.view.canvas.unbind("<ButtonPress-1>")
                self.view.canvas.unbind("<B1-Motion>")
                self.view.canvas.unbind("<ButtonRelease-1>")
                self.model.color = 'green'
                self.view.canvas.bind("<ButtonPress-1>", self.model.press)
                self.view.canvas.bind("<B1-Motion>", self.model.motion)
                self.view.canvas.bind("<ButtonRelease-1>", self.model.release)
                self.status.set("Next: 'Set area'")
            elif self.view.b3.get() == 0:
                self.view.b0.set(1)
                self.return_pointer()
        else:
            tkMessageBox.showinfo("PLASTron","Please load an image first")
            self.status.set("Please load an image first") 
            self.view.b3.set(0)

    def test_for_tag(self, item_id):
        test = False

        for item,tag in self.model.lines.iteritems():
            if tag[:2] == item_id:
                test = True

        return test

    def set_plastron_area(self):
        self.view.b0.set(1)
        self.return_pointer()


        if self.openScreen is True:
            tkMessageBox.showinfo("PLAStron    ", "Please load an image first")
            self.status.set("Load an image first") 

        elif self.test_for_tag('pb') == False:
            tkMessageBox.showinfo("PLASTron    ","outline plastron")
            self.status.set("outline plastron") 

        else:
            self.status.set("getting plastron area...")
            self.model.lineType = 'plastron'
            self.plastronArea = 0
               
            if self.plastronCoords == {}:
                  
                self.set_plastronCoords(0, 800)

                # make dict of plastron coords
                for item,tag in self.model.lines.iteritems():
                    if tag[:2] == 'pb':
                        coords = self.view.canvas.coords(item) 
                        if coords != []:
                            self.plastronCoords[round(coords[1])].append(round(coords[0]))
                            self.plastronCoords[round(coords[3])].append(round(coords[2]))                
                            restOfPoints = self.model.get_rest_of_points((round(coords[0]),round(coords[1])), 
                                                                         (round(coords[2]),round(coords[3])))
                            for p in restOfPoints:
                                self.plastronCoords[round(p[1])].append(round(p[0]))

                smoothCoords = self.smooth_area('plastron')   

                maxY =  0
                minY =  800

                for y, xvalue in self.plastronCoords.iteritems():
                    if xvalue != [] and y < minY:
                        minY = y
                    if xvalue != [] and y > maxY:
                        maxY = y

                # draw lines from left to right only to first point 
                for newY in range(minY, maxY + 1):
                    if smoothCoords.has_key(newY):
                        xlist = smoothCoords[newY]
                        if xlist != []:
                            xvalues = list(set(xlist[:]))
                            xvalues.sort()

                            for i in range(len(xvalues) - 1):
                                if i%2 == 0:
                                    point1 = xvalues[i], newY
                                    point2 = xvalues[i + 1], newY
                                    self.pf_tag += 1
                                    item = self.view.canvas.create_line(point1,point2,fill=self.model.color)
                                    self.model.lines[item] = "pf" + str(self.pf_tag)
                                    self.plastronArea += abs(point1[0] - point2[0])
                        else:
                            pass

                ## take fill away (comment the following lines to show plastron area)
                for item, tag in self.model.lines.iteritems():
                    if tag[:2] == 'pf':
                        self.view.canvas.delete(item)

                self.view.messageBoard.delete("plastron_area")
                item  = self.view.messageBoard.create_text(100,130,text="plastron = "+str(self.plastronArea),fill="white")
                self.view.messageBoard.itemconfig(item, tags = "plastron_area")
                self.status.set("Next: 'Outline softness'")
            else:
                tkMessageBox.showwarning("PLASTron    ","Plastron area can be set only one time")
                self.status.set("WARNING: Plastron area can be set only one time")
      
           
    def set_softness_area(self):
        self.view.b0.set(1)
        self.return_pointer()
        
        if self.openScreen is True:
            tkMessageBox.showinfo("PLASTron    ", "Load an image first")
            self.status.set("Load an image first") 

        elif self.test_for_tag('sb') == False:
            tkMessageBox.showinfo("PLASTron    ","outline softness")
            self.status.set("outline softness") 

        else:
            self.model.lineType = 'softness'
            self.softnessArea = 0

            if self.softnessCoords == {}:
                self.status.set("Setting softness area")
                self.view.messageBoard.delete("softness_area")
            else:
                self.status.set("Adding additional softness area")

            self.set_softnessCoords(0, 800)
   
            # make dict of plastron coords
            for item,tag in self.model.lines.iteritems():
                if tag[:2] == 'sb':
                    coords = self.view.canvas.coords(item) 
                    if coords != []:
                        self.softnessCoords[round(coords[1])].append(round(coords[0]))
                        self.softnessCoords[round(coords[3])].append(round(coords[2]))                
                        restOfPoints = self.model.get_rest_of_points((round(coords[0]),round(coords[1])), 
                                                                     (round(coords[2]),round(coords[3])))
                        for p in restOfPoints:
                            self.softnessCoords[round(p[1])].append(round(p[0]))

            smoothCoords = self.smooth_area('softness')   
            maxY =  0
            minY =  800

            for y, xvalue in self.softnessCoords.iteritems():
                if xvalue != [] and y < minY:
                    minY = y
                if xvalue != [] and y > maxY:
                    maxY = y
           
            # draw lines from left to right only to first point 
            for newY in range(minY, maxY + 1):
                if smoothCoords.has_key(newY):
                    xlist = smoothCoords[newY]
                    if xlist != []:
                        xvalues = list(set(xlist[:]))
                        xvalues.sort()
   
                        for i in range(len(xvalues) - 1):
                            if i%2 == 0:
                                point1 = xvalues[i], newY
                                point2 = xvalues[i + 1], newY
                                self.pf_tag += 1
                                item = self.view.canvas.create_line(point1,point2,fill=self.model.color)
                                self.model.lines[item] = "sf" + str(self.sf_tag)
                                self.softnessArea += abs(point1[0] - point2[0])
                    else:
                        pass

            self.view.messageBoard.delete("softness_area")
            item  = self.view.messageBoard.create_text(100,150,text="softness = "+ str(self.softnessArea),fill="white")
            self.view.messageBoard.itemconfig(item, tags = "softness_area")
            self.status.set("Next: 'save project'")

    def smooth_area(self, areaName):    
        smoothCoords = {}
        filterSize = 5
   
        if areaName == 'plastron':
            area = self.plastronCoords

        elif areaName == 'softness':
            area = self.softnessCoords
        else:
            print "ERROR: in smooth area input"
            sys.exit()
            
        for y, values in area.iteritems():
            if values != []:
                values = list(set(values))
                values.sort()
                values = array(values)
                newValues = values[:]

                ## removing unessary points 
                lastValue = 100
                for i in range(len(values)):
                    
                    if values[i] - lastValue < filterSize:
                        newValues[i] = 0

                    lastValue = values[i]
                        
                smoothCoords[y] = newValues[where(newValues != 0)]

        area = smoothCoords.copy()

        return smoothCoords

    def return_pointer(self):
        if self.view.b0.get() == 1:
            self.view.canvas.configure(cursor = "arrow")
            self.view.b1.set(0) 
            self.view.b3.set(0)
            self.view.b5.set(0)
            self.model.eraser = False
            self.view.canvas.unbind("<ButtonPress-1>")
            self.view.canvas.unbind("<B1-Motion>")
            self.view.canvas.unbind("<ButtonRelease-1>")

    def show_documentation(self):
        self.status.set("Loading documentation...")
        os.system("acroread " + "./UserManual/User_Manual_Plastron.pdf &")
         
    def calculate_relative_area(self):
        
        if self.openScreen is True:
            tkMessageBox.showinfo("PLASTron    ", "Load an image first")
            self.status.set("Load an image first") 

        elif self.test_for_tag('pb') == False:
            tkMessageBox.showinfo("PLASTron    ","outline plastron")
            self.status.set("outline plastron") 
        elif self.test_for_tag('sb') == False:
            tkMessageBox.showinfo("PLASTron    ","outline softness")
            self.status.set("outline softness") 

        elif self.test_for_tag('pf') == False:
            tkMessageBox.showinfo("PLASTron    ","set plastron")
            self.status.set("set plastron") 
        elif self.test_for_tag('sf') == False:
            tkMessageBox.showinfo("PLASTron    ","set softness")
            self.status.set("set softness") 
        else: 
            self.relativeArea = round(100.0 * (float(self.softnessArea) / float(self.plastronArea)), 4) 
            self.status.set("Calculating relative area") 
            self.view.messageBoard.delete("relative_area")
            item=self.view.messageBoard.create_text(100,170,text="relative area = "+str(self.relativeArea)+"%")
            self.view.messageBoard.itemconfig(item, fill = "white")
            self.view.messageBoard.itemconfig(item, tags = "relative_area")
            self.view.messageBoard.create_line((0,180),(200,180), fill = "white")
        

    def load_image(self): 
       
        self.openScreen = False
        import Image
        self.clear_plastron()
        self.clear_softness()
        self.set_variables()
        fileName = None
        fileName = askopenfilename(filetypes=[("allfiles","*")])
        fileName = os.path.realpath(fileName)
        if fileName == None or fileName == '':
            self.openScreen = True
        else:
            try:
                im = Image.open(fileName)
                im.thumbnail((800,600),Image.ANTIALIAS)         
            except:
                im = Image.open("green_sea_turtle.png")
                im.thumbnail((800,600),Image.ANTIALIAS)
                self.openScreen = True
                self.status.set("WARNING: File could not be opened")
                tkMessageBox.showwarning("PLASTron","File could not be opened") 
        try:
            self.view.mount_image(im)
            self.status.set("Next: 'outline plastron'")            
        except:
            self.openScreen = True
            self.status.set("WARNING: cannot open file")
            tkMessageBox.showwarning("PLASTron","opened file cannot be mounted")

    def enter_date(self):

        if self.openScreen is False:
            self.date = tkSimpleDialog.askstring("PLASTron", "Enter date")
            self.view.messageBoard.delete("date")
            self.view.messageBoard.create_text(100, 30, text = "Date = " + self.date, fill = "white", tags = "date")
        else:
            tkMessageBox.showinfo("PLASTron","Please load an image first")
            self.status.set("Please load an image first")     

    def enter_id(self):

        if self.openScreen is False:
            self.turtle_id = tkSimpleDialog.askstring("PLASTron", "Enter Turtle ID")
            self.view.messageBoard.delete("turtle_id")
            self.view.messageBoard.create_text(100, 50,text="Turtle ID = " + self.turtle_id,fill="white",tags="turtle_id")
        else:
            tkMessageBox.showinfo("PLASTron","Please load an image first")
            self.status.set("Please load an image first")     

    def enter_pid(self):
        self.project_id = tkSimpleDialog.askstring("PLASTron", "Enter Project ID")
        self.view.messageBoard.delete("project_id")
        self.view.messageBoard.create_text(100,70,text="Project ID = "+self.project_id,fill="white",tags="project_id") 

    def eraser(self):
        if self.openScreen is False:
            
            if self.view.b5.get() == 1:
                self.view.canvas.configure(cursor = "dotbox")
                self.model.eraser = True
                self.view.b0.set(0)
                self.view.b1.set(0)
                self.view.b3.set(0)
                self.view.canvas.unbind("<ButtonPress-1>")
                self.view.canvas.unbind("<B1-Motion>")
                self.view.canvas.unbind("<ButtonRelease-1>")
                self.view.canvas.bind("<ButtonPress-1>", self.model.press)
                self.view.canvas.bind("<B1-Motion>", self.model.motion)
                self.view.canvas.bind("<ButtonRelease-1>", self.model.release)
            elif self.view.b5.get() == 0:
                self.view.b0.set(1)
                self.return_pointer()         
        else:
            tkMessageBox.showinfo("PLASTron","Please load an image first")
            self.status.set("Please load an image first") 
            self.view.b5.set(0)

    def save_data(self):
        
        if self.currentFileName == None:
            tkMessageBox.showwarning("PLASTron    ", "Select 'save as' first") 
        elif self.date == None:
            tkMessageBox.showwarning("PLASTron    ", "Enter a date")  
        elif self.project_id == None:
            tkMessageBox.showwarning("PLASTron    ", "Enter a project ID") 
        elif self.turtle_id == None:
            tkMessageBox.showwarning("PLASTron    ", "Enter a turtle ID")
        elif self.plastronArea == 0:
            tkMessageBox.showwarning("PLASTron    ", "Find and set plastron area")
        elif self.softnessArea == 0:
            tkMessageBox.showwarning("PLASTron    ", "Find and set softness area")
        elif self.relativeArea == 0:
            tkMessageBox.showwarning("PLASTron    ", "Calculate relative area")      
        else:
            f = open (self.currentFileName,'a')
            f.write(self.project_id + ",")
            f.write(self.date       + ",")
            f.write(self.turtle_id  + ",")
            f.write(str(self.plastronArea) + ",")
            f.write(str(self.softnessArea) + ",")
            f.write(str(self.relativeArea) + "\n")
            f.close()
            self.status.set(" 'save' successful")

    def saveas_data(self):
    
        self.view.rightArea.delete("savefile")

        if self.openScreen is True:
            tkMessageBox.showwarning("PLASTron    ", "Load an image first")
        elif self.date == None:
            tkMessageBox.showwarning("PLASTron    ", "Enter a date")  
        elif self.project_id == None:
            tkMessageBox.showwarning("PLASTron    ", "Enter a project ID") 
        elif self.turtle_id == None:
            tkMessageBox.showwarning("PLASTron    ", "Enter a turtle ID")
        elif self.plastronArea == 0:
            tkMessageBox.showwarning("PLASTron    ", "Find and set plastron area")
        elif self.softnessArea == 0:
            tkMessageBox.showwarning("PLASTron    ", "Find and set softness area")
        elif self.relativeArea == 0:
            tkMessageBox.showwarning("PLASTron    ", "Calculate relative area")      
        else:
            fileName = asksaveasfilename(filetypes=[("csv files",".csv"),("allfiles","*")])
            fileName = os.path.realpath(fileName)
            fileName = re.sub("\.csv","",fileName)+".csv"
            f = None
            try:
                f = open(fileName, 'wb')
                self.currentFileName = fileName
            except:
                self.status.set("WARNING: could not open new file") 

            if f == None:
                pass
            else:
                f.write("project_id"  + ",")
                f.write("date"        + ",")
                f.write("turtle_id"   + ",")
                f.write("plas_area"   + ",")
                f.write("soft_area"   + ",")
                f.write("relt_area"   + "\n")
                f.close()
                self.save_data()

                if self.currentFileName != None:
                    self.view.rightArea.create_line((0,15),(200,15), fill = "white")
                    self.view.rightArea.create_text(100, 10, text="File", fill = "white")
                    self.view.rightArea.create_text(100, 30, text = "save = " + os.path.split(self.currentFileName)[-1],fill="white",tags="savefile")

            self.status.set(" 'save as' successful")


    def saveas_image(self):

        if self.openScreen is True:
            tkMessageBox.showwarning("PLASTron    ", "Load an image first")

        else:
            from PIL import Image
            fileName = asksaveasfilename(filetypes=[("jpg files",".jpg")])
            fileName = os.path.realpath(fileName)
            fileName = re.sub("\.jpg","",fileName)
            fileNamePS = fileName + ".ps"
            fileNameJpg = fileName + ".jpg"
            try:
                self.view.canvas.postscript(file=fileNamePS,colormode='color')
                im = Image.open(fileNamePS)
                im.save(fileNameJpg)
                os.remove(fileNamePS)
            except:
                tkMessageBox.showwarning("PLASTron","image was NOT saved succesfully")
                self.status.set("image save failed") 

    def clear_all(self):
                
        if self.openScreen is False:
            from PIL import Image,ImageTk
            self.view.b0.set(1)
            self.return_pointer()
            self.status.set("clearing all...")
            try:
                self.clear_plastron()
                self.clear_softness()
                self.set_variables()

                self.openScreen = True
                self.view.messageBoard.delete("project_id")
                self.view.messageBoard.delete("turtle_id")
                self.view.messageBoard.delete("date")
                im = Image.open("green_sea_turtle.png")
                im.thumbnail((800,600),Image.ANTIALIAS)
                self.i = ImageTk.PhotoImage(im)
                self.view.mount_image(im)
                self.status.set("All data cleared")
            except:
                tkMessageBox.showerror("PLASTron","Cannot clear all data")
                self.status.set("ERROR: cannot clear all data")         
        else:
            tkMessageBox.showinfo("PLASTron","Please load an image first")
            self.status.set("Please load an image first") 

    def clear_plastron(self):     
        if self.openScreen is False:

            self.status.set("clearing plastron...")
            self.view.b0.set(1)
            self.return_pointer()

            try:
                for item,tag in self.model.lines.iteritems():
                    if tag[:2] == "pb" or tag[:2] == "pf":
                        self.view.canvas.delete(item)

                self.view.messageBoard.delete("plastron_area")
                self.view.messageBoard.delete("relative_area")
                self.plastronCoords = {}
                self.plastronArea = 0
                self.relativeArea = 0
                self.status.set("plastron data cleared")

            except:
                tkMessageBox.showerror("PLASTron", "Cannot clear plastron area")
                self.status.set("ERROR: cannot clear plastron area")
    
        else:
            tkMessageBox.showinfo("PLASTron","Please load an image first")
            self.status.set("Please load an image first") 

    def clear_softness(self):
        if self.openScreen is False:
            self.view.b0.set(1)
            self.return_pointer()
            self.status.set("clearing softness...")
            try:
                for item,tag in self.model.lines.iteritems():
                    if tag[:2] == "sb" or tag[:2] == "sf":
                        self.view.canvas.delete(item)
                        
                self.view.messageBoard.delete("softness_area") 
                self.view.messageBoard.delete("relative_area")
                self.softnessCoords = {}
                self.softnessArea = 0
                self.relativeArea = 0
                self.status.set("softness data cleared")

            except:
                tkMessageBox.showerror("PLASTron", "Cannot clear softness area")
                self.status.set("ERROR: cannot clear softness area")
        else:
            tkMessageBox.showinfo("PLAStron","Please load an image first")
            self.status.set("Please load an image first")  
            

    def set_plastronCoords(self, minY, maxY):
        
        self.plastronCoords = {}
        for i in range(minY, maxY + 1):
            self.plastronCoords[i] = []

    def set_softnessCoords(self, minY, maxY):
        
        self.softnessCoords = {}
        for i in range(minY, maxY + 1):
            self.softnessCoords[i] = []
